@@ -98,13 +98,63 @@ sub render {
 	    }
 	}
     }
-    my $upem = 1000;
+    my $upem = 1000;		# as per PDF::API2
 
-    foreach my $fragment ( @{ $self->{_content} } ) {
+    my $draw_bg = sub {
+	my ( $fx, $nfx, $x, $y, $w ) = @_;
+	my $h = $bb[2];		# "border"
+	my $d = abs($h)/25;
 
-	if ( my $hd = $self->get_element_handler($fragment->{type}) ) {
+	# If first with background, extend a little to the left.
+	if ( $fx == 0 || !$self->{_content}->[$fx-1]->{bgcolor} ) {
+	    $x -= $d;
+	    $w += $d;
+	}
+	# If last with background, extend a little to the right.
+	if ( $fx == $nfx-1 || !$self->{_content}->[$fx+1]->{bgcolor} ) {
+	    $w += 2*$d;
+	}
+
+	# If next is a strut, followed by same bg color,
+	# have the background span the strut.
+	#### TODO: Span multiple struts.
+	my $delta;
+	for ( my $i = $fx+1; $i < $nfx; $i++ ) {
+	    if ( $self->{_content}->[$i]->{type} eq "strut" ) {
+		$delta //= 0;
+		$delta += $self->{_content}->[$i]->{width};
+	    }
+	    elsif ( defined($delta)
+		 && $self->{_content}->[$i]->{bgcolor}
+		 && $self->{_content}->[$i]->{bgcolor}
+		    eq $self->{_content}->[$fx]->{bgcolor} ) {
+		$w += $delta;
+		last;
+	    }
+	}
+	# Draw the background.
+	$text->textend;
+	my $gfx = $text;	# sanity
+	$gfx->save;
+	$gfx->fillcolor( $self->{_content}->[$fx]->{bgcolor} );
+	$gfx->linewidth(0);
+	$gfx->rectangle( $x, $y+$d, $x+$w, $y+$h-$d );
+	$text->fill;
+	$gfx->restore;
+	$text->textstart;
+    };
+
+    my $nfx = @{ $self->{_content} };
+    for ( my $fx = 0; $fx < $nfx; $fx++ ) {
+	my $fragment = $self->{_content}->[$fx];
+
+	if ( $fragment->{type} eq "strut" ) {
+	    $x += $fragment->{width};
+	}
+
+	elsif ( my $hd = $self->get_element_handler($fragment->{type}) ) {
 	    $text->textend;
-	    my $ab = $hd->render($fragment, $text, $x, $y-$bl);
+	    my $ab = $hd->render($fragment, $text, $x, $y-$bl)->{abox};
 	    $text->textstart;
 	    $x += $ab->[2];
 	}
@@ -136,23 +186,7 @@ sub render {
 	    $w += $_->{ax} for @$info;
 
 	    if ( $fragment->{bgcolor} ) {
-		my $y = $y0;
-		my $h = -$sz*($f->get_ascender-$f->get_descender)/$upem;
-		my $x = $x0;
-		$text->textend;
-		my $gfx = $text;
-		$gfx->save;
-		$gfx->fillcolor($fragment->{bgcolor});
-		$gfx->strokecolor($fragment->{bgcolor});
-		$gfx->linewidth(2);
-		$gfx->move($x, $y);
-		$gfx->line($x+$w, $y);
-		$gfx->line($x+$w, $y+$h);
-		$gfx->line($x, $y+$h);
-		$text->close;
-		$text->fillstroke;
-		$gfx->restore;
-		$text->textstart;
+		$draw_bg->( $fx, $nfx, $x0, $y0, $w );
 	    }
 
 	    foreach my $g ( @$info ) {
@@ -190,27 +224,9 @@ sub render {
 		my $w = $font->width($t) * $sz;
 
 		if ( $fragment->{bgcolor} ) {
-		    my ( $d0, $a0 );
-		    $d0 = $f->get_descender * $sz / $upem;
-		    $a0 = $f->get_ascender * $sz / $upem;
-		    my $y = $y0;
-		    my $h = -($a0-$d0);
-		    my $x = $x0;
-		    $text->textend;
-		    my $gfx = $text;
-		    $gfx->save;
-		    $gfx->fillcolor($fragment->{bgcolor});
-		    $gfx->strokecolor($fragment->{bgcolor});
-		    $gfx->linewidth(2);
-		    $gfx->move($x, $y);
-		    $gfx->line($x+$w, $y);
-		    $gfx->line($x+$w, $y+$h);
-		    $gfx->line($x, $y+$h);
-		    $text->close;
-		    $text->fillstroke;
-		    $gfx->restore;
-		    $text->textstart;
+		    $draw_bg->( $fx, $nfx, $x0, $y0, $w );
 		}
+
 		$text->font( $f->get_font, $sz );
 		$text->translate( $x, $y );
 		$text->text($t);
@@ -219,9 +235,9 @@ sub render {
 	}
 
 	next unless $x > $x0;
-
-	my $dw = 1000;
-	my $xh = $font->xheight;
+	# While PDF::API2 delivers font metrics in 1/1000s,
+	# underlinethickness and position are unscaled UPEM.
+	my $dw = $font->data->{upem} // 1000;
 
 	my @strikes;
 	if ( $fragment->{underline} && $fragment->{underline} ne 'none' ) {
@@ -242,9 +258,10 @@ sub render {
 
 	if ( $fragment->{strikethrough} ) {
 	    my $sz = $fragment->{size} || $self->{_currentsize};
+	    my $xh = $font->xheight / 1000;
 	    my $d = -( $f->{strikeline_position}
-		       ? $f->{strikeline_position}
-		       : 0.6*$xh ) * $sz/$dw;
+		       ? $f->{strikeline_position} / $dw
+		       : 0.6*$xh ) * $sz;
 	    my $h = ( $f->{strikeline_thickness}
 		      || $f->{underline_thickness}
 		      || $font->underlinethickness ) * $sz/$dw;
@@ -254,12 +271,13 @@ sub render {
 
 	if ( $fragment->{overline} && $fragment->{overline} ne 'none' ) {
 	    my $sz = $fragment->{size} || $self->{_currentsize};
+	    my $xh = $font->xheight / 1000;
 	    my $h = ( $f->{overline_thickness}
 		      || $f->{underline_thickness}
 		      || $font->underlinethickness ) * $sz/$dw;
 	    my $d = -( $f->{overline_position}
 		       ? $f->{overline_position} * $sz/$dw
-		       : $xh*$sz/$dw + 2*$h );
+		       : $xh*$sz + 2*$h );
 	    my $col = $fragment->{overline_color} // $fragment->{color};
 	    if ( $fragment->{overline} eq 'double' ) {
 		push( @strikes, [ $d-0.125*$h, $h * 0.75, $col ],
@@ -302,6 +320,7 @@ sub bbox {
     my ( $d, $a ) = (0) x 2;
     my ( $xMin, $xMax, $yMin, $yMax );
     my $dir;
+    $self->{_struts} = [];
 
     foreach ( @{ $self->{_content} } ) {
 
@@ -311,7 +330,21 @@ sub bbox {
 		  map { defined($_) ? sprintf("%.2f", $_) : "<undef>" }
 		  $xMin, $yMin, $xMax, $yMax ), "\n");
 
-	if ( my $hd = $self->get_element_handler($_->{type}) ) {
+	if ( $_->{type} eq "strut" ) {
+	    my @ab = ( 0, -($_->{desc}//0),
+		       $_->{width}//0, $_->{asc}//0 );
+	    my %s = %$_;
+	    delete($s{type});
+	    $s{_x} = $w;
+	    $s{_strut} = $_;
+	    push( @{ $self->{_struts} }, \%s );
+	    # Add to bbox but not to inkbox.
+	    $w += $ab[2];
+	    $a = $ab[3] if $ab[3] > $a;
+	    $d = $ab[1] if $ab[1] < $d;
+	}
+
+	elsif ( my $hd = $self->get_element_handler($_->{type}) ) {
 	    my @ab = @{$hd->bbox($_)->{abox}};
 	    $xMin //= $w + $ab[0] if $all;
 	    $xMax = $w + $ab[2];
@@ -387,7 +420,7 @@ sub bbox {
 	    if ( $all ) {
 		$_ -= $base for $e->{yMin}, $e->{yMax};
 		# Baseline oriented box.
-		$xMin //= $e->{xMin};
+		$xMin //= $w - $e->{wx} + $e->{xMin};
 		$yMin = $e->{yMin}
 		  if !defined($yMin) || $e->{yMin} < $yMin;
 		$yMax = $e->{yMax}
